Unit Btreetop;

{ btree1 -- a general unit for manipulating alphabetized strings
  on the heap }

{ Copyright 1989, by J. W. Rider  }

{ Modified for readability by Philip Howlett 1993 }

Interface

Type
  top_recptr = ^top_btrec;
  top_btrec = record
      upper,
      right,
      left       : top_recptr;
      counter    : integer;
      startword,
      Nextword   : string[15];
    end;

Procedure top_firstrec(var current : top_recptr);
Procedure top_lastrec(var current : top_recptr);
Procedure top_delrec(var root,current: top_recptr);
Function top_findrec(var current: top_recptr; start, next: string) : Boolean;
Procedure top_nextrec(var current: top_recptr);
Procedure top_prevrec(var current: top_recptr);
Function top_storec(var root: top_recptr; var start, next : string) : top_recptr;

Implementation

Procedure top_firstrec(var current : top_recptr);

{ returns the address of the first record in the btree branch
  from "current". if no btree has been entered, returns nil. }

Begin
  If current<>nil then
    While current^.left<>nil do
      current:=current^.left;
End;


Procedure top_lastrec(var current : top_recptr);

{ returns the address of the last record in the btree branch
  from "current". if no btree has been entered, returns nil. }

Begin
  If current<>nil then
    While current^.right<>nil do
      current:=current^.right;
End;


Procedure top_delrec(var root, current : top_recptr);

Var
  tr : top_recptr;

{ delete the current record }

Begin
  tr:=current;
  If tr<>nil then
    Begin

   { replace the current node with the right hand branches }

      If tr^.right<>nil then
        Begin
          current := tr^.right;
          top_firstrec(current);
          current^.left := tr^.left;
          tr^.right^.upper := tr^.upper;
          If tr^.left<>nil then
            tr^.left^.upper:=current;
          If tr^.upper<>nil then
            If tr^.upper^.left=tr then
              tr^.upper^.left := tr^.right
            Else
              tr^.upper^.right := tr^.right
          Else
            root:=tr^.right;
        End

   { replace the current node with the left hand branches }

      Else
        If tr^.left<>nil then
          Begin
            tr^.left^.upper  :=  tr^.upper;
            If tr^.upper<>nil then
              If tr^.upper^.right=tr then
                tr^.upper^.right := tr^.left
              Else
                tr^.upper^.left := tr^.left
            Else
              root := tr^.left;
          End

   { current node is a leaf }

        Else
          If tr^.upper<>nil then
            If tr^.upper^.right=tr then
              tr^.upper^.right := nil
            Else
              tr^.upper^.left := nil

   { current node is only node in tree }

          Else
            root:=nil;

   { always return the "parent" as current }

      current := tr^.upper;

   { free up the memory }

      freemem(tr,sizeof(top_btrec));
    End;
End;


Function top_findrec(var current : top_recptr; start, next : string) : boolean;

{ find the record that matches the string }

Begin
  top_findrec := false;
  While current<>nil do
    If (current^.startword = start) and (current^.nextword = next) then
      Begin
        top_findrec := true;
        exit;
      End
    Else
      If current^.startword>start then
        If current^.left<>nil then
          current := current^.left
        Else
          exit
      Else
        If current^.right<>nil then
          current := current^.right
        Else
          exit;
End;


Procedure top_nextrec(var current : top_recptr);

{ makes current the record following current record }

Begin
  If current<>nil then
    If current^.right<>nil then
      Begin
        current := current^.right;
        While current^.left<>nil do
          current := current^.left;
      End
    Else
      Begin
        While (current^.upper<>nil) and (current^.upper^.right=current) do
          current := current^.upper;
        If current^.upper<>nil then
          current := current^.upper
        Else
          current := nil;
      End;
End;


Procedure top_prevrec(var current : top_recptr);

{ makes current the record preceding current record }

Begin
  If current<>nil then
    If current^.left<>nil then
      Begin
        current := current^.left;
        While current^.right<>nil do
          current := current^.right;
      End
    Else
      Begin
        While (current^.upper<>nil) and (current^.upper^.left=current) do
          current := current^.upper;
        If current^.upper<>nil then
          current := current^.upper
        Else current := nil;
      End;
End;


Function top_storec(var root : top_recptr; var start, Next : string) : top_recptr;

{ stores a btree record }

Var
  current,
  newrec : top_recptr;

Function top_incrrec : boolean;

{ if record already exists, just increment its count '.c' }

Begin
  current := root;
  top_incrrec := true;
  If top_findrec(current,start, next) then
    inc(current^.counter)
  Else
    top_incrrec := false;
End;

Begin { procedure storec }

   { if the record already exists, just increment the count c }

  If not top_incrrec then

   { if there is enough room to store the record }

    If (maxavail>=(sizeof(top_btrec))) then
      Begin

   { allocate room for the record }

        Getmem(newrec,Sizeof(Top_btrec));
        newrec^.counter := 1;
        newrec^.right := nil;
        newrec^.left := nil;
        top_storec := newrec;

   { store the record into the btree }

        newrec^.startword := start;
        newrec^.nextword := next;
        newrec^.upper := current;
        If current<>nil then
          If current^.startword > start then
            current^.left := newrec
          Else
            current^.right := newrec
        Else
          root := newrec;
      End
    Else
      top_storec := nil
  Else
    top_storec := current;
End; { procedure storec }

End.
